function [p,dp]=legnorm(l,m,col)
%LEGNORM    fully normalized associated Legendre functions and its first
%           derivatives
%
%[p,dp]=legnorm(l,m,col)
%
%l      degree (vector)
%m      order (scalar)
%col    colatitude [deg] (vector)
%
%p, dp  fully normalized associated Legendre functions p and its first
%       derivatives dp, size(p)=size(dp)=[length(col),length(l)]
%
%2011-10-31, Muenchen, Michael Murboeck

if max(size(m))~=1 || rem(m,1)~=0 || m<0
    error('the order m must be a positive integer scalar')
end
if min(size(l))~=1 || any(rem(l,1)~=0) || any(l<m)
    error('all degrees l must be integer and larger than m')
end
if min(size(col))~=1
    error('col must not be a matrix')
end
col=col*pi/180;
if size(col,1)==1; col=col'; end
lmax=max(l);

t=cos(col);
y=sqrt(1-t.^2);
p00=ones(length(t),1);
dp00=zeros(length(t),1);
p=zeros(length(t),lmax-m+1);
dp=zeros(length(t),lmax-m+1);

switch m
    case 0
        plm3=sqrt(3)*t;             % P10
        plm2=sqrt(5)*(3*t.^2-1)/2;  % P20
        p(:,1:3)=[p00,plm3,plm2];
        dplm3=-y*wlmfun(1,0);
        dplm2=wlmfun(2,0)*(-y.*plm3+t.*dplm3);
        dp(:,1:3)=[dp00,dplm3,dplm2];
    case 1
        plm3=sqrt(3)*y;             % P11
        plm2=sqrt(15)*t.*y;         % P21
        p(:,1:2)=[plm3,plm2];
        dplm3=wlmfun(1,1)*t;
        dplm2=wlmfun(2,1)*(-y.*plm3+t.*dplm3);
        dp(:,1:2)=[dplm3,dplm2];
    otherwise
        plm3=zeros(size(t));        % Pm-1,m
        plm2=sqrt(15)*y.^2/2;       % P22
        dplm3=zeros(size(t));
        dplm2=wlmfun(2,2)*(sqrt(3)*t.*y+wlmfun(1,1)*y.*t);
        if m>2
            for n=4:m+1
                dplm2=wlmfun(n-1,n-1)*(t.*plm2+y.*dplm2);
                plm2=sqrt((2*n-1)/(2*n-2))*y.*plm2;
            end
        end
        p(:,1)=plm2;
        dp(:,1)=dplm2;
end

if lmax>max(2,m)
    for n=max(2,m)+2:lmax+1
        plm1=sqrt((2*n-1)/(n-m-1)*(2*n-3)/(n+m-1))*t.*plm2-...
            sqrt((2*n-1)/(2*n-5)*(n+m-2)/(n+m-1)*(n-m-2)/(n-m-1))*plm3;
        dplm1=wlmfun(n-1,m)*(-y.*plm2+t.*dplm2-dplm3/wlmfun(n-2,m));
        p(:,n-m)=plm1;
        dp(:,n-m)=dplm1;
        plm3=plm2; plm2=plm1;
        dplm3=dplm2; dplm2=dplm1;
    end
end

p=p(:,l-m+1);
dp=dp(:,l-m+1);

function w=wlmfun(l,m)
%WLMFUN     computes normalizing coefficients for Legendre Functions
%
%w=wlmfun(l,m)
%
%l, m   Degree (scalar) and order (scalar)
%
%w      normalizing coefficients (scalar)

if numel(l)~=1; error('l must be a scalar'); end
if numel(m)~=1; error('m must be a scalar'); end

if l==m    
    if l==0
        w=1;
    elseif l==1
        w=sqrt(3);
    else
        w=sqrt((2*l+1)/(2*l));
    end
else
    w=sqrt((2*l+1)/(l+m)*(2*l-1)/(l-m));
end
