function [scs,ncs,header,scst,ncst]=readicgem(filename)

% READICGEM reads potential coefficients in ICGEM-format from an ASCII
% file.
%
% HOW - [scs,ncs,header,scst,ncst]=readicgem(filename);
%
% IN  - filename : name and path of the model to compare
%
% OUT - scs      : coefficients in cs-format (Signal)
%     - ncs      : coef. errors in cs-format (Noise)
%     - header   : structure containing header information
%     - scst     : dot-coefficients in cs-format
%     - ncst     : dot-errors in cs-format
%
%--------------------------------------------------------------------------
% Martin Wermuth, IAPG, TU-Munich                                10/10/2005 
%--------------------------------------------------------------------------
% uses cs_format
%----------------------------------------------------------------------------
% revision history
%   - Michael Murboeck, IAPG, TU-Munich                          19/01/2010
%       separation of formal and calibrated errors
%       dot-coefficients
%       cs_format instead of ma2cs
%----------------------------------------------------------------------------
% remarks
%
%--------------------------------------------------------------------------


% Read Header
fid = fopen(filename);
product_type='';
modelname = '';
GM = 0;
ae = 0;
Lmax =0;
errors = '';
norm = '';
tide = '';
url='';
reference_epoch='';

s=fgets(fid);
while(strncmp(s, 'end_of_head', 11) == 0),
    if (strncmp(s, 'product_type', 12)), product_type = strtrim(s(13:end)); end;
    if (strncmp(s, 'modelname', 9)), modelname = strtrim(s(10:end)); end;
    if (strncmp(s, 'earth_gravity_constant', 22)), GM = str2num(s(23:end)); end;
    if (strncmp(s, 'radius', 6)), ae = str2num(s(7:end)); end;
    if (strncmp(s, 'max_degree', 10)), Lmax = str2double(s(11:end)); end;
    if (strncmp(s, 'errors', 6)), errors = strtrim(s(7:end)); end;
    if (strncmp(s, 'error', 5)), errors = strtrim(s(7:end)); end;
    if (strncmp(s, 'norm', 4)), norm = strtrim(s(5:end)); end;
    if (strncmp(s, 'tide_system', 11)), tide = strtrim(s(12:end)); end;     
    if (strncmp(s, 'url', 3)), url = strtrim(s(4:end)); end;
    if (strncmp(s, 'reference_epoch', 15)), reference_epoch = strtrim(s(16:end)); end;
    s=fgets(fid);    
end
header = struct('product_type',product_type,'modelname', modelname,...
    'earth_gravity_constant',GM,'radius',ae,'max_degree',Lmax,...
    'errors',errors,'norm',norm,'tide_system',tide,'url',url,'reference_epoch',reference_epoch);

if strcmpi(header.errors,'no') || strcmpi(header.errors,'not included')
    % read coefficients
    Clm = zeros(Lmax+1);
    Slm = zeros(Lmax+1);

    Clm_t = zeros(Lmax+1);
    Slm_t = zeros(Lmax+1);
    
    Lmax_t=0;

    s=fgets(fid);
    if strcmpi(s(1),' '); s=s(2:end); end
    
    while (s>=0)
        n=str2num(s(5:end));
        if strcmpi(s(1:3),'dot'),
           l=n(1)+1;
           m=n(2)+1;
           Clm_t(l,m) = n(3);
           Slm_t(l,m) = n(4);

           Lmax_t=max(Lmax_t,n(1));
        elseif strcmpi(s(1:3),'gfc'),
           l=n(1)+1;
           m=n(2)+1;
           Clm(l,m) = n(3);
           Slm(l,m) = n(4);

        end       
        if strcmpi(s(1:4),'gfct'), header.reference_epoch = n(end); end
        s=fgets(fid);
        if strcmpi(s(1),' '); s=s(2:end); end
    end
    fclose(fid);

    Clm_t = Clm_t(1:Lmax_t+1,1:Lmax_t+1);
    Slm_t = Slm_t(1:Lmax_t+1,1:Lmax_t+1);

    % transform coefficients to cs Format
    % cs = ma2cs(Clm, Slm);
    % sigma = ma2cs(sigClm, sigSlm);
    scs=cs_format(Clm,Slm);
    scst=cs_format(Clm_t,Slm_t);
    ncs=[];
    ncst=[];
else
    % read coefficients
    Clm = zeros(Lmax+1);
    Slm = zeros(Lmax+1);
    sigClm = zeros(Lmax+1);
    sigSlm = zeros(Lmax+1);

    Clm_t = zeros(Lmax+1);
    Slm_t = zeros(Lmax+1);
    sigClm_t = zeros(Lmax+1);
    sigSlm_t = zeros(Lmax+1);
    Lmax_t=0;

    s=fgets(fid);
    if strcmpi(s(1),' '); s=s(2:end); end
    while (s>=0)
        
        n=str2num(s(5:end));
        if strcmpi(s(1:3),'dot'),
           l=n(1)+1;
           m=n(2)+1;
           Clm_t(l,m) = n(3);
           Slm_t(l,m) = n(4);
           if (strcmpi(header.errors,'calibrated_and_formal')),
               sigClm_t(l,m) = n(7);
               sigSlm_t(l,m) = n(8);
               header.errors='formal';
           else
               sigClm_t(l,m) = n(5);
               sigSlm_t(l,m) = n(6);
           end
           Lmax_t=max(Lmax_t,n(1));
        elseif strcmpi(s(1:3),'gfc'),
           l=n(1)+1;
           m=n(2)+1;
           Clm(l,m) = n(3);
           Slm(l,m) = n(4);
           if (strcmpi(header.errors,'calibrated_and_formal')),
               sigClm(l,m) = n(7);
               sigSlm(l,m) = n(8);
           else
               sigClm(l,m) = n(5);
               sigSlm(l,m) = n(6);
           end
        end       
        if strcmpi(s(1:4),'gfct'), header.reference_epoch = n(end); end
        s=fgets(fid);
        if strcmpi(s(1),' '); s=s(2:end); end
    end
    fclose(fid);

    Clm_t = Clm_t(1:Lmax_t+1,1:Lmax_t+1);
    Slm_t = Slm_t(1:Lmax_t+1,1:Lmax_t+1);
    sigClm_t = sigClm_t(1:Lmax_t+1,1:Lmax_t+1);
    sigSlm_t = sigSlm_t(1:Lmax_t+1,1:Lmax_t+1);

    % transform coefficients to cs Format
    % cs = ma2cs(Clm, Slm);
    % sigma = ma2cs(sigClm, sigSlm);
    scs=cs_format(Clm,Slm);
    ncs=cs_format(sigClm,sigSlm);
    scst=cs_format(Clm_t,Slm_t);
    ncst=cs_format(sigClm_t,sigSlm_t);
end