function global_grid=shs(gco,fun,lmax,col,lon,GM,ae,alt)
%SHS    spherical harmonic (SH) synthesison on a global grid
%
%global_grid=shs(gco,fun,lmax,col,lon,GM,ae)
%
%gco    gravity coefficients CS-format, (lmax+1,lmax+1)-matrix with Clm in the lower triangular
%	and Slm in the upper triangular
%fun    gravity functionals, vector with max. 5 entries, default: 1
%       1   geoid undulation [m]
%       2   gravity anomaly [mGal]
%       3   vertical gravity gradient [E]
%       4   total water storage [mm water equivalent]=[kg/m^2]
%       5   no dimensioning [-]
%       6   gravity disturbance [mGal]
%       7   pressure [Pa]=[kg/m/s^2]
%       8   vertical crustal deformation [m]
%       e.g. fun=[1,3] for output in geoid heights and gradients
%lmax   max. SH degree, default: lmax=size(gco,1)-1
%col    co-latitude vector [deg], default: col=0.5:179.5
%lon    longitude vector [deg], default: lon=0.5:359.5
%GM     Earth gravity constant times Earth mass [m^3/s^2], default: GM=3.986005e14
%ae     Earth radius [m], default: ae=6378137
%alt    Satellite altitude [m], default: alt=0 (earth surface)
%
%global_grid    output grids, (length(col),length(lon),length(fun))-matrix containing
%       the functionals defined in fun with the same sequence

        if nargin<8; alt=0; end % PA
if nargin<7; ae=6.378137e6; end
if nargin<6; GM=3.986005e14; end
G=6.67259e-11;
if nargin<5 || isempty(lon); lon=0.5:359.5; end
if size(lon,1)>1; lon=lon'; end
if nargin<4 || isempty(col); col=0.5:179.5; end
[r,c]=size(gco);
if r~=c; error('gco must be a quadratic matrix'); end
if nargin<3 || isempty(lmax); lmax=r-1; end
if lmax>(r-1) || lmax<0 || rem(lmax,1)~=0
    error('lmax must be a positive integer scalar smaller than size(gco,1)')
end
if nargin<2; fun=1; end
fun=unique(fun);
if any(~ismember(fun,1:8)); error('all entries in fun must be in 1:8'); end

l_vec=0:lmax;
m_vec=(0:lmax)';
cos_coef=tril(gco);
sin_coef=(triu(gco,1))';
sin_coef=[sin_coef(:,end),sin_coef(:,1:end-1)];
cos_mlon=cos(m_vec*lon*pi/180);
sin_mlon=sin(m_vec*lon*pi/180);

global_grid=zeros(length(col),length(lon),length(fun));
for i=1:length(fun)
    Am=zeros(length(col),lmax+1);
    Bm=zeros(length(col),lmax+1);
    for k=1:length(m_vec)
        p_fun=legnorm(l_vec(k:end),m_vec(k),col);
        attfact=(ae/(ae+alt)).^(l_vec(k:end)+1); % PA
        switch fun(i)
            case 1
                c_fun=ae;
                l_fun=ones(size(l_vec(k:end)));
%                 l_fun=ones(size(l_vec(k:end))).*attfact; % PA
            case 2
                c_fun=1e5*GM/ae^2;
%                 l_fun=(l_vec(k:end)-1);
                l_fun=(l_vec(k:end)-1);
            case 3
                c_fun=1e9*GM/ae^3;
                l_fun=(l_vec(k:end)+1).*(l_vec(k:end)+2);
%                 l_fun=(l_vec(k:end)+1).*(l_vec(k:end)+2).*attfact; % PA
            case 4
				ks=load('loadlove_farrell.dat');
				ks=[0,ks(1:lmax,3)'];
                c_fun=GM/G/4/pi/ae^2;
                l_fun=(2*l_vec(k:end)+1)./(1+ks(k:end));
%                 l_fun=(2*l_vec(k:end)+1)./(1+ks(k:end)).*attfact; % PA
            case 5
                c_fun=1;
                l_fun=ones(size(l_vec(k:end)));
%                 l_fun=ones(size(l_vec(k:end))).*attfact; % PA
            case 6
                c_fun=1e5*GM/ae^2;
                l_fun=(l_vec(k:end)+1);
%                 l_fun=(l_vec(k:end)+1).*attfact; % PA
            case 7
                g0=9.81;
				ks=load('loadlove_farrell.dat');
				ks=[0,ks(1:lmax,3)'];
                c_fun=g0*GM/G/4/pi/ae^2;
                l_fun=(2*l_vec(k:end)+1)./(1+ks(k:end));
%                 l_fun=(2*l_vec(k:end)+1)./(1+ks(k:end)).*attfact; % PA
            case 8
				ks=load('loadlove_farrell.dat');
                hs=[0,0,ks(2:lmax,2)'];
				ks=[0,ks(1:lmax,3)'];
                c_fun=ae;
                l_fun=hs(k:end)./(1+ks(k:end));
%                 l_fun=hs(k:end)./(1+ks(k:end)).*attfact; % PA
            otherwise
                error('wrong entry in fun')
        end
        l_fun=l_fun.*attfact; % PA
        Am(:,k)=p_fun.*repmat(l_fun,length(col),1)*cos_coef(k:lmax+1,k);
        if m_vec(k)~=0
            Bm(:,k)=p_fun.*repmat(l_fun,length(col),1)*sin_coef(k:lmax+1,k);
        end
    end
    global_grid(:,:,i)=c_fun*(Am*cos_mlon+Bm*sin_mlon);
end

%     switch fun(i)
%         case 1
%             global_grid(:,:,i)=ae*(Am*cos_mlon+Bm*sin_mlon);
%         case 2
%             Am_ga=Am.*repmat(l_vec'-1,length(col),1);
%             Bm_ga=Bm.*repmat(l_vec'-1,length(col),1);
%             global_grid(:,:,i)=1e5*GM/ae^2*(Am_ga*cos_mlon+Bm_ga*sin_mlon);
%         case 3
%             Am_vg=Am.*repmat(l_vec'+1,length(col),1).*repmat(l_vec'+2,length(col),1);
%             Bm_vg=Bm.*repmat(l_vec'+1,length(col),1).*repmat(l_vec'+2,length(col),1);
%             global_grid(:,:,i)=1e9*GM/ae^3*(Am_vg*cos_mlon+Bm_vg*sin_mlon);
%         case 4
%             ks=load('loadlove_farrell.dat');
%             ks=[0;ks(1:lmax,3)];
%             Am_sm=Am.*repmat(2*l_vec'+1,length(col),1)./repmat(1+ks',length(col),1);
%             Bm_sm=Bm.*repmat(2*l_vec'+1,length(col),1)./repmat(1+ks',length(col),1);
%             global_grid(:,:,i)=(GM/G/4/pi/ae^2)*(Am_sm*cos_mlon+Bm_sm*sin_mlon);
%         case 5
%             global_grid(:,:,i)=Am*cos_mlon+Bm*sin_mlon;
%         case 6
%             Am_ga=Am.*repmat(l_vec'+1,length(col),1);
%             Bm_ga=Bm.*repmat(l_vec'+1,length(col),1);
%             global_grid(:,:,i)=1e5*GM/ae^2*(Am_ga*cos_mlon+Bm_ga*sin_mlon);
%         case 7
%             g0=9.81;
%             ks=load('loadlove_farrell.dat');
%             ks=[0;ks(1:lmax,3)];
%             Am_sm=Am.*repmat(2*l_vec'+1,length(col),1)./repmat(1+ks',length(col),1);
%             Bm_sm=Bm.*repmat(2*l_vec'+1,length(col),1)./repmat(1+ks',length(col),1);
%             global_grid(:,:,i)=g0*(GM/G/4/pi/ae^2)*(Am_sm*cos_mlon+Bm_sm*sin_mlon);
%         otherwise
%             error('wrong entry in fun')
%     end
% end